import eyed3
import requests
import json
import os
from dzr_import.tracks import get_session_id, get_tracks_ids, get_user_data


def get_tracks_metadata(tracks, api_token, session_id):
    res = requests.post(f'https://www.deezer.com/ajax/gw-light.php?method=song.getListData&input=3&api_version=1.0&api_token={api_token}', 
                       cookies={'sid': session_id},
                       headers={'Content-Type': 'application/json', 'Accept':'application/json'},
                       data=json.dumps({'sng_ids': tracks}))
    
    data = list(map(lambda track: {
        'artist': track['ART_NAME'], 
        'release_date': track['PHYSICAL_RELEASE_DATE'],
        'title': track['SNG_TITLE'],
        'album': track['ALB_TITLE'],
        'track_number': track['TRACK_NUMBER'],
        'album_art': track['ALB_PICTURE'],
        }, res.json()['results']['data']))
    
    return data


def add_track_metadata(track, metadata, path):
        audiofile = eyed3.load(f"{path}/{track[0]} - {track[1]}.mp3")
        if audiofile.tag == None:
            audiofile.initTag()

        # Add basic tags
        audiofile.tag.title = metadata["title"]
        audiofile.tag.album = metadata["album"]
        audiofile.tag.artist = metadata["artist"]
        audiofile.tag.release_date = metadata["release_date"]
        audiofile.tag.track_num = metadata["track_number"]
        audiofile.tag.file_name = f"{track[0]} - {track[1]}.mp3"

        image_url = f"https://e-cdn-images.dzcdn.net/images/cover/{metadata['album_art']}/1024x1024-000000-100-0-0.jpg"
        image_data = requests.get(image_url, timeout=60).content
        audiofile.tag.images.set(3, image_data, "image/jpeg")
        audiofile.tag.save()
        

def tag_tracks(tracks):
    tracks_ids = get_tracks_ids(tracks)
    session_id = get_session_id()
    user_token, user_license, api_token = get_user_data(session_id)
    tracks_data = get_tracks_metadata(tracks_ids, api_token, session_id)
    for track, metadata in zip(tracks, tracks_data):
        add_track_metadata(track, metadata, "output")
    


import exiftool
import requests
import os

from dzr_import.dec import calcbfkey, decryptfile
from dzr_import.tag import tag_tracks
from dzr_import.tracks import get_cdns, get_session_id, get_tracks_data, get_tracks_ids, get_user_data


def download_tracks(cdns, tracks):
    for cdn, track in zip(cdns, tracks):
        res = requests.get(cdn, stream=True)
        with open(f"output/{track[0]} - {track[1]}_encrypted.mp3", 'wb') as f:
            for chunk in res.iter_content(chunk_size=1024):
                if chunk:
                    f.write(chunk)


def decrypt_tracks(tracks, ids):
    for track, id in zip(tracks, ids):
        with open(f"output/{track[0]} - {track[1]}_encrypted.mp3", 'rb') as fh, open(f"output/{track[0]} - {track[1]}.mp3", 'wb') as fo:
            decryptfile(fh, calcbfkey(id).encode('utf-8'), fo)
        os.remove(f"output/{track[0]} - {track[1]}_encrypted.mp3")

def run():
    files = [f"tracks/{track}" for track in os.listdir('tracks')]
    tracks = []
    with exiftool.ExifToolHelper() as et:
        metadata = et.get_metadata(files)
        tracks = [(d['Vorbis:Artist'], d['Vorbis:Title']) for d in metadata]

    tracks_ids = get_tracks_ids(tracks)
    session_id = get_session_id()
    user_token, user_license, api_token = get_user_data(session_id)
    tracks_data = get_tracks_data(tracks_ids, api_token, session_id)
    cdns = get_cdns(list(map(lambda track: track['token'], tracks_data)), user_license)
    download_tracks(cdns, tracks)
    decrypt_tracks(tracks, list(map(lambda track: track['id'], tracks_data)))
    tag_tracks(tracks)


import requests
import json

def get_session_id():
    res = requests.get('https://www.deezer.com/ajax/gw-light.php?method=deezer.ping&api_version=1.0&api_token')
    return res.json()['results']['SESSION']

def get_user_data(session_id):
    res = requests.get('https://www.deezer.com/ajax/gw-light.php?method=deezer.getUserData&input=3&api_version=1.0&api_token=', cookies={'sid': session_id})
    user_token = res.json()['results']['USER_TOKEN']
    user_license = res.json()['results']['USER']['OPTIONS']['license_token']
    api_token = res.json()['results']['checkForm']

    return user_token, user_license, api_token

def get_tracks_ids(tracks):
    res = map(lambda track: requests.get('https://api.deezer.com/search?q=artist:"{}" track:"{}"'.format(*track)), tracks)
    
    return list(map(lambda r: r.json()['data'][0]['id'], res))

def get_tracks_data(tracks, api_token, session_id):
    res = requests.post(f'https://www.deezer.com/ajax/gw-light.php?method=song.getListData&input=3&api_version=1.0&api_token={api_token}', 
                       cookies={'sid': session_id},
                       headers={'Content-Type': 'application/json', 'Accept':'application/json'},
                       data=json.dumps({'sng_ids': tracks}))
    
    data = list(map(lambda track: {'id': track['SNG_ID'], 'token': track['TRACK_TOKEN']}, res.json()['results']['data']))
    
    return data
    
def get_cdns(tokens, user_license):
    res = requests.post('https://media.deezer.com/v1/get_url',
                        headers={'Content-Type': 'application/json', 'Accept':'application/json'},
                        data=json.dumps({
                            'track_tokens': tokens,
                            'license_token': user_license,
                            'media': [{
                                'type': 'FULL', 'formats': [{
                                    'cipher': 'BF_CBC_STRIPE',
                                    'format': 'MP3_128',
                                }]
                                }],
                            }))
    
    return list(map(lambda data: data['media'][0]['sources'][0]['url'], res.json()['data']))